@@ -1,8 +1,6 @@
 #ifndef Rcpp11_h
 #define Rcpp11_h
 
-// #define RCPP_DEBUG_LEVEL 1
-
 #include <Rcpp/platform.h>
 
 #include <cmath>
@@ -11,6 +9,7 @@
 #include <initializer_list>
 #include <unordered_map>
 #include <unordered_set>
+#include <climits>
 
 #include <Rcpp/macros/macros.h>
 #include <Rcpp/R.h>
@@ -43,7 +42,7 @@
 #include <numeric>
 #include <algorithm>
 #include <complex>
-#include <limits>
+#include <climits>
 #include <typeinfo>
 #include <tuple>
 #include <utility>
@@ -52,119 +51,126 @@
 
 #include <Rcpp/api/CRTP.h>
 #include <Rcpp/complex.h>
+#include <Rcpp/utils/tinyformat.h>
 
 #include <Rcpp/exceptions.h>
+#include <Rcpp/internal/interrupt.h>
 #include <Rcpp/Demangler.h>
+#include <Rcpp/utils/describe.h>
+
+#if defined(RCPP11_EXPERIMENTAL_PARALLEL)
+#include <thread>
+#endif
+
+#include <Rcpp/utils/parallel/parallel.h>
 
 namespace Rcpp{
     
-    inline const char* short_file_name(const char* file) {
+    inline bool is_null(SEXP x){ 
+        return Rf_isNull(x) ;   
+    }
+    inline bool is_object(SEXP x){ 
+        return Rf_isObject(x) ;   
+    }
+    inline bool inherits(SEXP x, const char* klass){ 
+        return Rf_inherits(x, klass) ;   
+    }
+    
+    inline std::string short_file_name(const char* file) {
         std::string f(file) ;
-        return f.substr( f.find_last_of("/") + 1 ).c_str() ;
+        size_t pos = f.find_last_of("/") ;
+        if( pos == std::string::npos ) return f ;
+        return f.substr( pos + 1 ) ;
+    }
+
+    inline bool derives_from( SEXP cl, const std::string& clazz ){
+        if(cl == R_NilValue) return false ;
+
+        // simple test for exact match
+        if( ! clazz.compare( CHAR(STRING_ELT(cl, 0)) ) ) return true ;
+
+        SEXP containsSym = Rf_install("contains");
+        Shield<SEXP> contains = R_do_slot(R_getClassDef(CHAR(Rf_asChar(cl))),containsSym);
+        SEXP res = Rf_getAttrib(contains,R_NamesSymbol );
+        if(res == R_NilValue) return false ;
+
+        return std::any_of(
+            VECTOR_PTR(res), VECTOR_PTR(res) + LENGTH(res),
+            [&](SEXP s){ return clazz == CHAR(s) ; }
+        ) ;
     }
 
     class String ;
-    template <typename CLASS> class PreserveStorage ;
-    template <typename CLASS> class NoProtectStorage ;
-    
-    template <int RTYPE, template <class> class StoragePolicy = PreserveStorage> 
-    class Vector ;
-    
-    template <int RTYPE, template <class> class StoragePolicy = PreserveStorage> 
-    class Matrix ;
-    
+    class PreserveStorage ;
+    class NoProtectStorage ;
+
+    template <int RTYPE, typename Storage = PreserveStorage> class Vector ;
+    template <int RTYPE, typename Storage = PreserveStorage> class Matrix ;
+
     typedef Vector<STRSXP> CharacterVector ;
-    typedef Vector<VECSXP> List ; 
-    typedef Vector<EXPRSXP> ExpressionVector ; 
-    
-    RCPP_API_CLASS_DECL(RObject) 
-    RCPP_API_CLASS_DECL(Language) 
-    RCPP_API_CLASS_DECL(Pairlist)
-    RCPP_API_CLASS_DECL(Environment)
-    RCPP_API_CLASS_DECL(Promise)
-    RCPP_API_CLASS_DECL(WeakReference)
-    RCPP_API_CLASS_DECL(S4)
-    RCPP_API_CLASS_DECL(Formula)
-    RCPP_API_CLASS_DECL(Reference)
-    RCPP_API_CLASS_DECL(Function)
-    
-    template < template <class> class StoragePolicy > class Symbol_Impl ;
-    typedef Symbol_Impl<NoProtectStorage> Symbol ;
-    
-    template < template <class> class StoragePolicy > class DataFrame_Impl ;
+    typedef Vector<VECSXP> List ;
+    typedef Vector<EXPRSXP> ExpressionVector ;
+
+    template <typename Storage> class RObject_Impl ;
+    template <typename Storage> class Language_Impl ;
+    template <typename Storage> class Pairlist_Impl ;
+    template <typename Storage> class Environment_Impl ;
+    template <typename Storage> class Promise_Impl ;
+    template <typename Storage> class WeakReference_Impl ;
+    template <typename Storage> class S4_Impl ;
+    template <typename Storage> class Formula_Impl ;
+    template <typename Storage> class Reference_Impl ;
+    template <typename Storage> class Function_Impl ;
+    template <typename Storage> class DataFrame_Impl ;
+    template <typename Storage> class Symbol_Impl ;
+
+    typedef RObject_Impl<PreserveStorage> RObject ;
+    typedef Language_Impl<PreserveStorage> Language ;
+    typedef Pairlist_Impl<PreserveStorage> Pairlist ;
+    typedef Environment_Impl<PreserveStorage> Environment ;
+    typedef Promise_Impl<PreserveStorage> Promise ;
+    typedef WeakReference_Impl<PreserveStorage> WeakReference ;
+    typedef S4_Impl<PreserveStorage> S4 ;
+    typedef Formula_Impl<PreserveStorage> Formula ;
+    typedef Reference_Impl<PreserveStorage> Reference ;
+    typedef Function_Impl<PreserveStorage> Function ;
     typedef DataFrame_Impl<PreserveStorage> DataFrame ;
+    typedef Symbol_Impl<NoProtectStorage> Symbol ;
     
 }
 namespace Rcpp{
-    inline SEXP Rcpp_PreserveObject(SEXP x){ 
-        if( x != R_NilValue ) {
-            R_PreserveObject(x); 
-        }
-        return x ;
-    }
-
-    inline void Rcpp_ReleaseObject(SEXP x){
-        if (x != R_NilValue) {
-            R_ReleaseObject(x); 
-        }
-    }
-
-    inline SEXP Rcpp_ReplaceObject(SEXP x, SEXP y){
-        if( x == R_NilValue ){
-            Rcpp_PreserveObject( y ) ;    
-        } else if( y == R_NilValue ){
-            Rcpp_ReleaseObject( x ) ;
-        } else {
-            // if we are setting to the same SEXP as we already have, do nothing 
-            if (x != y) {
-                
-                // the previous SEXP was not NULL, so release it 
-                Rcpp_ReleaseObject(x);
-                
-                // the new SEXP is not NULL, so preserve it 
-                Rcpp_PreserveObject(y);
-            }
-        }
-        return y ;
-    } 
+    class Na_Proxy ;
     
     template <typename T> T as( SEXP ) ;
 }
 
 #include <Rcpp/internal/na.h>
 #include <Rcpp/traits/traits.h>
+#include <Rcpp/sugar/functional/functional.h>
 #include <Rcpp/Named.h>
 
 #include <Rcpp/internal/caster.h>
 #include <Rcpp/internal/r_vector.h>
 #include <Rcpp/r_cast.h>
 
-#include <Rcpp/internal/export.h>
 #include <Rcpp/internal/r_coerce.h>
-#include <Rcpp/as.h>
+#include <Rcpp/as/forward.h>
 #include <Rcpp/InputParameter.h>
 #include <Rcpp/is.h>
 
-#include <Rcpp/vector/VectorBase.h>
-#include <Rcpp/vector/MatrixBase.h>
-
 #include <Rcpp/wrap/wrap.h>
 
 #include <Rcpp/internal/Proxy_Iterator.h>
-#include <Rcpp/internal/const_Proxy_Iterator.h>
-#include <Rcpp/internal/converter.h>
-
-#include <Rcpp/sugar/sugar_forward.h>
 
 #include <Rcpp/longlong.h>
 #include <Rcpp/transient_vector.h>
 
 #include <Rcpp/registration/registration.h>
 
-#include <Rcpp/exceptions.h>
-
-#include <Rcpp/proxy/proxy.h>
 #include <Rcpp/storage/storage.h>
+#include <Rcpp/proxy/GenericProxy.h>
+#include <Rcpp/Symbol.h>
+#include <Rcpp/proxy/proxy.h>
 
 #include <Rcpp/storage/PreserveStorage.h>
 #include <Rcpp/storage/NoProtectStorage.h>
@@ -185,10 +191,11 @@ namespace Rcpp{
 #include <Rcpp/Evaluator.h>
 
 #include <Rcpp/Vector.h>
+#include <Rcpp/ListOf.h>
+
 #include <Rcpp/sugar/nona/nona.h>
 
 #include <Rcpp/XPtr.h>
-#include <Rcpp/Symbol.h>
 #include <Rcpp/Function.h>
 #include <Rcpp/Language.h>
 #include <Rcpp/Pairlist.h>
@@ -216,4 +223,6 @@ namespace Rcpp{
 
 #include <Rcpp/Timer.h>
 
+namespace Rcpp11 = Rcpp ;
+
 #endif
