@@ -3,21 +3,23 @@
 
 namespace Rcpp{ 
     
-    template < template <class> class StoragePolicy>
+    template <typename Storage>
     class Dots_Impl {
     public:
-        typedef Environment_Impl<StoragePolicy> Environment ;
-        typedef Promise_Impl<StoragePolicy> Promise ;
+        typedef Environment_Impl<Storage> Environment ;
+        typedef Promise_Impl<Storage> Promise ;
         
         Dots_Impl( Environment env){
             SEXP dots = env.find("...") ;
-            while(dots != R_NilValue){
-                promises.push_back(CAR(dots)) ;
-                dots = CDR(dots);
+            if( dots != R_MissingArg ){ 
+                while(dots != R_NilValue){
+                    promises.push_back(CAR(dots)) ;
+                    dots = CDR(dots);
+                }
             }
         }
         
-        inline int size() const {
+        inline R_xlen_t size() const {
             return promises.size() ;    
         }
         
@@ -34,7 +36,7 @@ namespace Rcpp{
         std::vector<Promise> promises ;
     } ;
     
-    typedef Dots_Impl<NoProtectStorage> Dots ; 
+    typedef Dots_Impl<NoProtectStorage> Dots ;
     
 }
 
