@@ -1,11 +1,11 @@
 #ifndef RCPP_SUGAR_H
 #define RCPP_SUGAR_H
 
-#include <Rcpp/sugar/tools/iterator.h>
-#include <Rcpp/sugar/block/block.h>
+#include <Rcpp/sugar/iterators/iterators.h>
 
-#include <Rcpp/sugar/operators/operators.h>
 #include <Rcpp/sugar/functions/functions.h>
+#include <Rcpp/sugar/operators/operators.h>
+#include <Rcpp/sugar/lazy/lazy.h>
 
 #include <Rcpp/sugar/matrix/matrix_functions.h>
 
