@@ -2,12 +2,8 @@
 #define Rcpp_wrap_wrap_h
  
 #include <Rcpp/wrap/make_charsexp.h>
-#include <Rcpp/wrap/primitive_range_wrap.h>
-#include <Rcpp/wrap/range_wrap.h>
-#include <Rcpp/wrap/primitive_wrap.h>
 
 #include <Rcpp/wrap/wrap_impl.h>
-#include <Rcpp/wrap/rowmajor.h>
 
 #include <Rcpp/wrap/Wrapper.h>     
 #include <Rcpp/wrap/PrimitiveWrapper.h>     
@@ -16,6 +12,7 @@
 #include <Rcpp/wrap/ContainerWrapper.h>
 #include <Rcpp/wrap/SexpConvertibleWrapper.h>
 #include <Rcpp/wrap/SugarExpressionWrapper.h>
+#include <Rcpp/wrap/LazyVectorWrapper.h>
 
 #include <Rcpp/wrap/wrap_type.h>     
 
